@@ -3,27 +3,31 @@
  * Copyright © 2012-2013 Collabora, Ltd.
  * Copyright © 2013 Jason Ekstrand
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "config.h"
 
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <wayland-util.h>
@@ -385,6 +389,7 @@ frame_set_title(struct frame *frame, const char *title)
 	free(frame->title);
 	frame->title = dup;
 
+	frame->geometry_dirty = 1;
 	frame->status |= FRAME_STATUS_REPAINT;
 
 	return 0;
@@ -426,7 +431,7 @@ frame_resize_inside(struct frame *frame, int32_t width, int32_t height)
 	struct theme *t = frame->theme;
 	int decoration_width, decoration_height, titlebar_height;
 
-	if (frame->title)
+	if (frame->title || !wl_list_empty(&frame->buttons))
 		titlebar_height = t->titlebar_height;
 	else
 		titlebar_height = t->width;
@@ -467,7 +472,7 @@ frame_refresh_geometry(struct frame *frame)
 	if (!frame->geometry_dirty)
 		return;
 
-	if (frame->title)
+	if (frame->title || !wl_list_empty(&frame->buttons))
 		titlebar_height = t->titlebar_height;
 	else
 		titlebar_height = t->width;
@@ -782,27 +787,27 @@ frame_pointer_button(struct frame *frame, void *data,
 	return location;
 }
 
-void
+enum theme_location
 frame_touch_down(struct frame *frame, void *data, int32_t id, int x, int y)
 {
 	struct frame_touch *touch = frame_touch_get(frame, data);
 	struct frame_button *button = frame_find_button(frame, x, y);
 	enum theme_location location;
 
-	if (id > 0)
-		return;
-
-	if (touch && button) {
-		touch->button = button;
-		frame_button_press(touch->button);
-		return;
-	}
-
 	location = theme_get_location(frame->theme, x, y,
 				      frame->width, frame->height,
 				      frame->flags & FRAME_FLAG_MAXIMIZED ?
 				      THEME_FRAME_MAXIMIZED : 0);
 
+	if (id > 0)
+		return location;
+
+	if (touch && button) {
+		touch->button = button;
+		frame_button_press(touch->button);
+		return location;
+	}
+
 	switch (location) {
 	case THEME_LOCATION_TITLEBAR:
 		frame->status |= FRAME_STATUS_MOVE;
@@ -820,6 +825,7 @@ frame_touch_down(struct frame *frame, void *data, int32_t id, int x, int y)
 	default:
 		break;
 	}
+	return location;
 }
 
 void
@@ -836,6 +842,86 @@ frame_touch_up(struct frame *frame, void *data, int32_t id)
 	}
 }
 
+enum theme_location
+frame_double_click(struct frame *frame, void *data,
+		   uint32_t btn, enum frame_button_state state)
+{
+	struct frame_pointer *pointer = frame_pointer_get(frame, data);
+	struct frame_button *button;
+	enum theme_location location = THEME_LOCATION_EXTERIOR;
+
+	location = theme_get_location(frame->theme, pointer->x, pointer->y,
+				      frame->width, frame->height,
+				      frame->flags & FRAME_FLAG_MAXIMIZED ?
+				      THEME_FRAME_MAXIMIZED : 0);
+
+	button = frame_find_button(frame, pointer->x, pointer->y);
+
+	if (location != THEME_LOCATION_TITLEBAR || btn != BTN_LEFT)
+		return location;
+
+	if (state == FRAME_BUTTON_PRESSED) {
+		if (button)
+			frame_button_press(button);
+		else
+			frame->status |= FRAME_STATUS_MAXIMIZE;
+	} else if (state == FRAME_BUTTON_RELEASED) {
+		if (button)
+			frame_button_release(button);
+	}
+
+	return location;
+}
+
+void
+frame_double_touch_down(struct frame *frame, void *data, int32_t id,
+			int x, int y)
+{
+	struct frame_touch *touch = frame_touch_get(frame, data);
+	struct frame_button *button = frame_find_button(frame, x, y);
+	enum theme_location location;
+
+	if (touch && button) {
+		touch->button = button;
+		frame_button_press(touch->button);
+		return;
+	}
+
+	location = theme_get_location(frame->theme, x, y,
+				      frame->width, frame->height,
+				      frame->flags & FRAME_FLAG_MAXIMIZED ?
+				      THEME_FRAME_MAXIMIZED : 0);
+
+	switch (location) {
+	case THEME_LOCATION_TITLEBAR:
+		frame->status |= FRAME_STATUS_MAXIMIZE;
+		break;
+	case THEME_LOCATION_RESIZING_TOP:
+	case THEME_LOCATION_RESIZING_BOTTOM:
+	case THEME_LOCATION_RESIZING_LEFT:
+	case THEME_LOCATION_RESIZING_RIGHT:
+	case THEME_LOCATION_RESIZING_TOP_LEFT:
+	case THEME_LOCATION_RESIZING_TOP_RIGHT:
+	case THEME_LOCATION_RESIZING_BOTTOM_LEFT:
+	case THEME_LOCATION_RESIZING_BOTTOM_RIGHT:
+		frame->status |= FRAME_STATUS_RESIZE;
+		break;
+	default:
+		break;
+	}
+}
+
+void
+frame_double_touch_up(struct frame *frame, void *data, int32_t id)
+{
+	struct frame_touch *touch = frame_touch_get(frame, data);
+
+	if (touch && touch->button) {
+		frame_button_release(touch->button);
+		frame_touch_destroy(touch);
+	}
+}
+
 void
 frame_repaint(struct frame *frame, cairo_t *cr)
 {
@@ -852,7 +938,7 @@ frame_repaint(struct frame *frame, cairo_t *cr)
 
 	cairo_save(cr);
 	theme_render_frame(frame->theme, cr, frame->width, frame->height,
-			   frame->title, flags);
+			   frame->title, &frame->buttons, flags);
 	cairo_restore(cr);
 
 	wl_list_for_each(button, &frame->buttons, link)
